# Copyright (c) 2020 - 2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from datetime import datetime
from subprocess import Popen, PIPE
import argparse
import os
import shutil
import sys
import platform

__author__ = "Kiriti Nagesh Gowda"
__copyright__ = "Copyright 2018 - 2024, AMD MIVisionX - Vision Test Full Report"
__license__ = "MIT"
__version__ = "1.4.0"
__maintainer__ = "Kiriti Nagesh Gowda"
__email__ = "mivisionx.support@amd.com"
__status__ = "Shipping"


def shell(cmd):
    p = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    output = p.communicate()[0][0:-1]
    return output


def write_formatted(output, f):
    f.write("````\n")
    f.write("%s\n\n" % output)
    f.write("````\n")


def strip_libtree_addresses(lib_tree):
    return lib_tree


# Import arguments
parser = argparse.ArgumentParser()
parser.add_argument('--runvx_directory',    type=str, default='/opt/rocm/bin',
                    help='RunVX Executable Directory - optional (default:/opt/rocm/bin)')
parser.add_argument('--hardware_mode',      type=str, default='CPU',
                    help='OpenVX Vision Function Target - optional (default:CPU [options:CPU/GPU])')
parser.add_argument('--list_tests',         type=str, default='NO',
                    help='List Vision Performance Tests - optional (default:NO [options:NO/Yes])')
parser.add_argument('--test_filter',        type=int, default=0,
                    help='Vision Performance Test Filter - optional (default:0 [range:1 - N])')
parser.add_argument('--num_frames',         type=int, default=1000,
                    help='Run Test for X number of frames - optional (default:1000 [range:1 - N])')
parser.add_argument('--functionality',      type=str, default='Yes',
                    help='Vision Functionality Tests Enabled - optional (default:YES [options:NO/YES])')
parser.add_argument('--backend_type',       type=str, default='CPU',
                    help='Backend type - optional (default:CPU [options:CPU/HIP/OCL])')
parser.add_argument('--profiling',          type=str, default='NO',
                    help='GPU profiling with rocprof - optional (default:NO [options:YES/NO])')
parser.add_argument('--width',              type=int, default='1920',
                    help='Image width for Vision Performance tests - optional (default:1920) [range: 1 - 7680]')
parser.add_argument('--height',             type=int, default='1080',
                    help='Image height for Vision Performance tests - optional (default:1080) [range: 1 - 7680]')
parser.add_argument('--perf_counters',        type=str, default='NO',
                    help='Collect performance counters with rocprof - optional (default:NO [options:YES/NO])')


args = parser.parse_args()

runvxDir = args.runvx_directory
hardwareMode = args.hardware_mode.upper()
listTest = args.list_tests.upper()
testFilter = args.test_filter
numFrames = args.num_frames
backendType = args.backend_type.upper()
profilingOption = args.profiling.upper()
width = args.width
height = args.height
widthDiv2 = int(width / 2)
heightDiv2 = int(height / 2)
perfCounters = args.perf_counters.upper()

# OpenVX Vision Functions 1080P
openvxNodes = [
    # Arithmetic kernels
    ('AbsDiff_U8_U8U8',                           'org.khronos.openvx.absdiff uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb image:' + str(width) + ',' + str(height) + ',U008'),
    ('AbsDiff_S16_S16S16_Sat',                    'org.khronos.openvx.absdiff uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb image:' + str(width) + ',' + str(height) + ',S016'),
    ('Add_U8_U8U8_Wrap',                          'org.khronos.openvx.add uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',U008'),
    ('Add_U8_U8U8_Sat',                           'org.khronos.openvx.add uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !SATURATE image:' + str(width) + ',' + str(height) + ',U008'),
    ('Add_S16_U8U8_Wrap',                         'org.khronos.openvx.add uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',S016'),
    ('Add_S16_S16U8_Wrap',                        'org.khronos.openvx.add uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',S016'),
    ('Add_S16_S16U8_Sat',                         'org.khronos.openvx.add uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !SATURATE image:' + str(width) + ',' + str(height) + ',S016'),
    ('Add_S16_S16S16_Wrap',                       'org.khronos.openvx.add uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',S016'),
    ('Add_S16_S16S16_Sat',                        'org.khronos.openvx.add uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb !SATURATE image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sub_U8_U8U8_Wrap',                          'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',U008'),
    ('Sub_U8_U8U8_Sat',                           'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !SATURATE image:' + str(width) + ',' + str(height) + ',U008'),
    ('Sub_S16_U8U8_Wrap',                         'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sub_S16_S16U8_Wrap',                        'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sub_S16_S16U8_Sat',                         'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb !SATURATE image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sub_S16_U8S16_Wrap',                        'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sub_S16_U8S16_Sat',                         'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb !SATURATE image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sub_S16_S16S16_Wrap',                       'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb !WRAP image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sub_S16_S16S16_Sat',                        'org.khronos.openvx.subtract uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb !SATURATE image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_U8_U8U8_Wrap_Trunc',                    'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',U008,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !WRAP !ROUND_POLICY_TO_ZERO image:' + str(width) + ',' + str(height) + ',U008'),
    ('Mul_U8_U8U8_Wrap_Round',                    'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',U008,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !WRAP !ROUND_POLICY_TO_NEAREST_EVEN image:' + str(width) + ',' + str(height) + ',U008'),
    ('Mul_U8_U8U8_Sat_Trunc',                     'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',U008,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !SATURATE !ROUND_POLICY_TO_ZERO image:' + str(width) + ',' + str(height) + ',U008'),
    ('Mul_U8_U8U8_Sat_Round',                     'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',U008,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !SATURATE !ROUND_POLICY_TO_NEAREST_EVEN image:' + str(width) + ',' + str(height) + ',U008'),
    ('Mul_S16_U8U8_Wrap_Trunc',                   'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',U008,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !WRAP !ROUND_POLICY_TO_ZERO image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_U8U8_Wrap_Round',                   'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',U008,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !WRAP !ROUND_POLICY_TO_NEAREST_EVEN image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_U8U8_Sat_Trunc',                    'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',U008,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !SATURATE !ROUND_POLICY_TO_ZERO image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_U8U8_Sat_Round',                    'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',U008,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !SATURATE !ROUND_POLICY_TO_NEAREST_EVEN image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_S16U8_Wrap_Trunc',                  'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',S016,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !WRAP !ROUND_POLICY_TO_ZERO image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_S16U8_Wrap_Round',                  'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',S016,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !WRAP !ROUND_POLICY_TO_NEAREST_EVEN image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_S16U8_Sat_Trunc',                   'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',S016,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !SATURATE !ROUND_POLICY_TO_ZERO image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_S16U8_Sat_Round',                   'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',S016,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xbb scalar:FLOAT32,1.0 !SATURATE !ROUND_POLICY_TO_NEAREST_EVEN image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_S16S16_Wrap_Trunc',                 'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',S016,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',S016,0xbb scalar:FLOAT32,1.0 !WRAP !ROUND_POLICY_TO_ZERO image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_S16S16_Wrap_Round',                 'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',S016,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',S016,0xbb scalar:FLOAT32,1.0 !WRAP !ROUND_POLICY_TO_NEAREST_EVEN image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_S16S16_Sat_Trunc',                  'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',S016,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',S016,0xbb scalar:FLOAT32,1.0 !SATURATE !ROUND_POLICY_TO_ZERO image:' + str(width) + ',' + str(height) + ',S016'),
    ('Mul_S16_S16S16_Sat_Round',                  'org.khronos.openvx.multiply uniform-image:' + str(width) + ',' + str(height) + ',S016,0xaa uniform-image:' + \
     str(width) + ',' + str(height) + ',S016,0xbb scalar:FLOAT32,1.0 !SATURATE !ROUND_POLICY_TO_NEAREST_EVEN image:' + str(width) + ',' + str(height) + ',S016'),
    ('Magnitude_S16_S16S16',                      'org.khronos.openvx.magnitude uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb image:' + str(width) + ',' + str(height) + ',S016'),
    ('Phase_U8_S16S16',                           'org.khronos.openvx.phase uniform-image:' + str(width) + ',' + str(height) + \
     ',S016,0xaa uniform-image:' + str(width) + ',' + str(height) + ',S016,0xbb image:' + str(width) + ',' + str(height) + ',U008'),
    ('WeightedAverage_U8_U8U8',                   'org.khronos.openvx.weighted_average uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa scalar:FLOAT32,0.25 uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb image:' + str(width) + ',' + str(height) + ',U008'),
    # logical kernels
    ('And_U8_U8U8',                               'org.khronos.openvx.and uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb image:' + str(width) + ',' + str(height) + ',U008'),
    ('Or_U8_U8U8',                                'org.khronos.openvx.or uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb image:' + str(width) + ',' + str(height) + ',U008'),
    ('Xor_U8_U8U8',                               'org.khronos.openvx.xor uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb image:' + str(width) + ',' + str(height) + ',U008'),
    ('Not_U8_U8',                                 'org.khronos.openvx.not uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xaa image:' + str(width) + ',' + str(height) + ',U008'),
    # Color kernels
    ('Lut_U8_U8',                                 'org.khronos.openvx.table_lookup uniform-image:' + str(width) + \
     ',' + str(height) + ',U008,0xaa lut:UINT8,256 image:' + str(width) + ',' + str(height) + ',U008'),
    ('ColorDepth_U8_S16_Wrap',                    'org.khronos.openvx.convertdepth uniform-image:' + str(width) + \
     ',' + str(height) + ',S016,0xaa image:' + str(width) + ',' + str(height) + ',U008 !WRAP scalar:INT32,1'),
    ('ColorDepth_U8_S16_Sat',                     'org.khronos.openvx.convertdepth uniform-image:' + str(width) + \
     ',' + str(height) + ',S016,0xaa image:' + str(width) + ',' + str(height) + ',U008 !SATURATE scalar:INT32,1'),
    ('ColorDepth_S16_U8',                         'org.khronos.openvx.convertdepth uniform-image:' + str(width) + \
     ',' + str(height) + ',U008,0xaa image:' + str(width) + ',' + str(height) + ',S016 !WRAP scalar:INT32,1'),
    ('ChannelExtract_U8_U16_Pos0',                'org.khronos.openvx.channel_extract uniform-image:' + str(width) + ',' + str(height) + \
     ',YUYV !CHANNEL_Y image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U16_Pos1',                'org.khronos.openvx.channel_extract uniform-image:' + str(width) + ',' + str(height) + \
     ',YUYV !CHANNEL_Y image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U24_Pos0',                'org.khronos.openvx.channel_extract uniform-image:' + \
     str(width) + ',' + str(height) + ',RGB2,0xaabbcc !CHANNEL_R image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U24_Pos1',                'org.khronos.openvx.channel_extract uniform-image:' + \
     str(width) + ',' + str(height) + ',RGB2,0xaabbcc !CHANNEL_G image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U24_Pos2',                'org.khronos.openvx.channel_extract uniform-image:' + \
     str(width) + ',' + str(height) + ',RGB2,0xaabbcc !CHANNEL_B image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U32_Pos0_UYVY',           'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',UYVY,0xaabbcc !CHANNEL_U image:' + str(widthDiv2) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U32_Pos1_YUYV',           'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',YUYV,0xaabbcc !CHANNEL_U image:' + str(widthDiv2) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U32_Pos2_UYVY',           'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',UYVY,0xaabbcc !CHANNEL_V image:' + str(widthDiv2) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U32_Pos3_YUYV',           'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',YUYV,0xaabbcc !CHANNEL_V image:' + str(widthDiv2) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U32_Pos0_RGBX',           'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',RGBA,0xaabbccdd !CHANNEL_R image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U32_Pos1_RGBX',           'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',RGBA,0xaabbccdd !CHANNEL_G image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U32_Pos2_RGBX',           'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',RGBA,0xaabbccdd !CHANNEL_B image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8_U32_Pos3_RGBX',           'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',RGBA,0xaabbccdd !CHANNEL_A image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8U8U8_U24',                 'org.khronos.openvx.channel_extract uniform-image:' + \
     str(width) + ',' + str(height) + ',RGB2,0xaabbcc !CHANNEL_R image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8U8U8_U32',                 'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',RGBA,0xaabbccdd !CHANNEL_R image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelExtract_U8U8U8U8_U32',               'org.khronos.openvx.channel_extract uniform-image:' + str(width) + \
     ',' + str(height) + ',RGBA,0xaabbccdd !CHANNEL_R image:' + str(width) + ',' + str(height) + ',U008'),
    ('ChannelCombine_U32_U8U8U8U8_RGBX',          'org.khronos.openvx.channel_combine uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa uniform-image:' + str(width) + ',' + str(height) + ',U008,0xbb uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xcc uniform-image:' + str(width) + ',' + str(height) + ',U008,0xdd image:' + str(width) + ',' + str(height) + ',RGBA'),
    ('ColorConvert_RGB_RGBX',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',RGBA,0xaabbccdd image:' + str(width) + ',' + str(height) + ',RGB2'),
    ('ColorConvert_RGB_UYVY',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',UYVY,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGB2'),
    ('ColorConvert_RGB_YUYV',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',YUYV,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGB2'),
    ('ColorConvert_RGB_IYUV',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',IYUV,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGB2'),
    ('ColorConvert_RGB_NV12',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',NV12,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGB2'),
    ('ColorConvert_RGB_NV21',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',NV21,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGB2'),
    ('ColorConvert_RGBX_RGB',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',RGB2,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGBA'),
    ('ColorConvert_RGBX_UYVY',                    'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',UYVY,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGBA'),
    ('ColorConvert_RGBX_YUYV',                    'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',YUYV,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGBA'),
    ('ColorConvert_RGBX_IYUV',                    'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',IYUV,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGBA'),
    ('ColorConvert_RGBX_NV12',                    'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',NV12,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGBA'),
    ('ColorConvert_RGBX_NV21',                    'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',NV21,0xaabbcc image:' + str(width) + ',' + str(height) + ',RGBA'),
    ('ColorConvert_IYUV_RGB',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',RGB2,0xaabbcc image:' + str(width) + ',' + str(height) + ',IYUV'),
    ('ColorConvert_IYUV_RGBX',                    'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',RGBA,0xaabbccdd image:' + str(width) + ',' + str(height) + ',IYUV'),
    ('FormatConvert_IYUV_UYVY',                   'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',UYVY,0xaabbcc image:' + str(width) + ',' + str(height) + ',IYUV'),
    ('FormatConvert_IYUV_YUYV',                   'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',YUYV,0xaabbcc image:' + str(width) + ',' + str(height) + ',IYUV'),
    ('ColorConvert_NV12_RGB',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',RGB2,0xaabbcc image:' + str(width) + ',' + str(height) + ',NV12'),
    ('ColorConvert_NV12_RGBX',                    'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',RGBA,0xaabbccdd image:' + str(width) + ',' + str(height) + ',NV12'),
    ('FormatConvert_NV12_UYVY',                   'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',UYVY,0xaabbcc image:' + str(width) + ',' + str(height) + ',NV12'),
    ('FormatConvert_NV12_YUYV',                   'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',YUYV,0xaabbcc image:' + str(width) + ',' + str(height) + ',NV12'),
    ('ColorConvert_YUV4_RGB',                     'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',RGB2,0xaabbcc image:' + str(width) + ',' + str(height) + ',YUV4'),
    ('ColorConvert_YUV4_RGBX',                    'org.khronos.openvx.color_convert uniform-image:' + \
     str(width) + ',' + str(height) + ',RGBA,0xaabbccdd image:' + str(width) + ',' + str(height) + ',YUV4'),
    ('FormatConvert_IUV_UV12',                    'org.khronos.openvx.color_convert uniform-image:' + str(width) + ',' + str(height) + \
     ',NV12,0xaabbcc image:' + str(width) + ',' + str(height) + ',IYUV'),
    ('FormatConvert_UV12_IUV',                    'org.khronos.openvx.color_convert uniform-image:' + str(width) + ',' + str(height) + \
     ',IYUV,0xaabbcc image:' + str(width) + ',' + str(height) + ',NV12'),
    ('FormatConvert_UV_UV12',                     'org.khronos.openvx.color_convert uniform-image:' + str(width) + ',' + str(height) + \
     ',NV12,0xaabbcc image:' + str(width) + ',' + str(height) + ',YUV4'),
    ('ScaleUp2x2_U8_U8',                          'org.khronos.openvx.color_convert uniform-image:' + str(width) + ',' + str(height) + \
     ',IYUV,0xaabbcc image:' + str(width) + ',' + str(height) + ',YUV4'),
    # filter kernels
    ('Box_U8_U8_3x3',                             'org.khronos.openvx.box_3x3 uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xaa image:' + str(width) + ',' + str(height) + ',U008'),
    ('Dilate_U8_U8_3x3',                          'org.khronos.openvx.dilate_3x3 uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xaa image:' + str(width) + ',' + str(height) + ',U008'),
    ('Erode_U8_U8_3x3',                           'org.khronos.openvx.erode_3x3 uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xaa image:' + str(width) + ',' + str(height) + ',U008'),
    ('Median_U8_U8_3x3',                          'org.khronos.openvx.median_3x3 uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xaa image:' + str(width) + ',' + str(height) + ',U008'),
    ('Gaussian_U8_U8_3x3',                        'org.khronos.openvx.gaussian_3x3 uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xaa image:' + str(width) + ',' + str(height) + ',U008'),
    ('ScaleGaussianHalf_U8_U8_3x3',               'org.khronos.openvx.halfscale_gaussian uniform-image:' + str(width) + \
     ',' + str(height) + ',U008,0xaa image:' + str(widthDiv2) + ',' + str(heightDiv2) + ',U008 scalar:INT32,3'),
    ('ScaleGaussianHalf_U8_U8_5x5',               'org.khronos.openvx.halfscale_gaussian uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa image:' + str(widthDiv2) + ',' + str(heightDiv2) + ',U008 scalar:INT32,5'),
    ('Convolve_U8_U8_3x3',                        'org.khronos.openvx.custom_convolution uniform-image:' + str(width) + ',' + \
     str(height) + ',U008,0xaa "convolution:3,3:INIT,{-1;-1;-1;-1;16;-1;-1;-1;-1}" image:' + str(width) + ',' + str(height) + ',U008'),
    ('Convolve_S16_U8_3x3',                       'org.khronos.openvx.custom_convolution uniform-image:' + str(width) + ',' + \
     str(height) + ',U008,0xaa "convolution:3,3:INIT,{-1;-1;-1;-1;16;-1;-1;-1;-1}" image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sobel_S16S16_U8_3x3_GXY',                   'org.khronos.openvx.sobel_3x3 uniform-image:' + str(width) + ',' + str(height) + \
     ',U008,0xaa image:' + str(width) + ',' + str(height) + ',S016 image:' + str(width) + ',' + str(height) + ',S016'),
    ('Sobel_S16_U8_3x3_GX',                       'org.khronos.openvx.sobel_3x3 uniform-image:' + \
     str(width) + ',' + str(height) + ',U008,0xaa image:' + str(width) + ',' + str(height) + ',S016'),
    # Statistical kernels
    ('Threshold_U8_U8_Binary',                    'org.khronos.openvx.threshold uniform-image:' + str(width) + ',' + \
     str(height) + ',U008,0xaa threshold:BINARY,U008,U008:INIT,127 image:' + str(width) + ',' + str(height) + ',U008'),
    ('Threshold_U8_U8_Range',                     'org.khronos.openvx.threshold uniform-image:' + str(width) + ',' + \
     str(height) + ',U008,0xaa threshold:RANGE,U008,U008:INIT,100,200 image:' + str(width) + ',' + str(height) + ',U008'),
    ('Threshold_U8_S16_Binary',                   'org.khronos.openvx.threshold uniform-image:' + str(width) + ',' + \
     str(height) + ',S016,0xaa threshold:BINARY,S016,U008:INIT,127 image:' + str(width) + ',' + str(height) + ',U008'),
    ('Threshold_U8_S16_Range',                    'org.khronos.openvx.threshold uniform-image:' + str(width) + ',' + \
     str(height) + ',S016,0xaa threshold:RANGE,S016,U008:INIT,100,200 image:' + str(width) + ',' + str(height) + ',U008'),
    # geometric kernels
    ('ScaleImage_U8_U8_Nearest',                  'org.khronos.openvx.scale_image image:' + str(width) + \
     ',' + str(height) + ',U008 image:' + str(widthDiv2) + ',' + str(heightDiv2) + ',U008 !NEAREST_NEIGHBOR'),
    ('ScaleImage_U8_U8_Bilinear',                 'org.khronos.openvx.scale_image image:' + str(width) + \
     ',' + str(height) + ',U008 image:' + str(widthDiv2) + ',' + str(heightDiv2) + ',U008 !BILINEAR'),
    ('ScaleImage_U8_U8_Bilinear_Replicate',       'org.khronos.openvx.scale_image image:' + str(width) + ',' + \
     str(height) + ',U008 image:' + str(widthDiv2) + ',' + str(heightDiv2) + ',U008 !BILINEAR attr:BORDER_MODE:REPLICATE'),
    ('ScaleImage_U8_U8_Bilinear_Constant',        'org.khronos.openvx.scale_image image:' + str(width) + ',' + \
     str(height) + ',U008 image:' + str(widthDiv2) + ',' + str(heightDiv2) + ',U008 !BILINEAR attr:BORDER_MODE:CONSTANT,0'),
    ('ScaleImage_U8_U8_Area',                     'org.khronos.openvx.scale_image image:' + str(width) + \
     ',' + str(height) + ',U008 image:' + str(widthDiv2) + ',' + str(heightDiv2) + ',U008 !AREA'),
    ('WarpAffine_U8_U8_Nearest',                  'org.khronos.openvx.warp_affine image:' + str(width) + ',' + str(height) + \
     ',U008 "matrix:FLOAT32,2,3:INIT,{0.25;0;0;0.5;20;20}" !NEAREST_NEIGHBOR image:' + str(width) + ',' + str(height) + ',U008'),
    ('WarpAffine_U8_U8_Nearest_Constant',         'org.khronos.openvx.warp_affine image:' + str(width) + ',' + str(height) + \
     ',U008 "matrix:FLOAT32,2,3:INIT,{0.25;0;0;0.5;20;20}" !NEAREST_NEIGHBOR image:' + str(width) + ',' + str(height) + ',U008 attr:BORDER_MODE:CONSTANT,0'),
    ('WarpAffine_U8_U8_Bilinear',                 'org.khronos.openvx.warp_affine image:' + str(width) + ',' + str(height) + \
     ',U008 "matrix:FLOAT32,2,3:INIT,{0.25;0;0;0.5;20;20}" !BILINEAR image:' + str(width) + ',' + str(height) + ',U008'),
    ('WarpAffine_U8_U8_Bilinear_Constant',        'org.khronos.openvx.warp_affine image:' + str(width) + ',' + str(height) + \
     ',U008 "matrix:FLOAT32,2,3:INIT,{0.25;0;0;0.5;20;20}" !BILINEAR image:' + str(width) + ',' + str(height) + ',U008 attr:BORDER_MODE:CONSTANT,0'),
    # TBD -- Failure on GFX 1030 -- TURN OFF for OpenCL Backend
    #('WarpPerspective_U8_U8_Nearest',             'org.khronos.openvx.warp_perspective image:' + str(width) + ',' + str(height) + \
    # ',U008 "matrix:FLOAT32,3,3:INIT,{2;0.1;0;2;1.9;0;-1200;-360;1}" !NEAREST_NEIGHBOR image:' + str(width) + ',' + str(height) + ',U008'),
    #('WarpPerspective_U8_U8_Nearest_Constant',    'org.khronos.openvx.warp_perspective image:' + str(width) + ',' + str(height) + \
    # ',U008 "matrix:FLOAT32,3,3:INIT,{2;0.1;0;2;1.9;0;-1200;-360;1}" !NEAREST_NEIGHBOR image:' + str(width) + ',' + str(height) + ',U008 attr:BORDER_MODE:CONSTANT,0'),
    #('WarpPerspective_U8_U8_Bilinear',            'org.khronos.openvx.warp_perspective image:' + str(width) + ',' + str(height) + \
    # ',U008 "matrix:FLOAT32,3,3:INIT,{2;0.1;0;2;1.9;0;-1200;-360;1}" !BILINEAR image:' + str(width) + ',' + str(height) + ',U008'),
    #('WarpPerspective_U8_U8_Bilinear_Constant',   'org.khronos.openvx.warp_perspective image:' + str(width) + ',' + str(height) + \
    # ',U008 "matrix:FLOAT32,3,3:INIT,{2;0.1;0;2;1.9;0;-1200;-360;1}" !BILINEAR image:' + str(width) + ',' + str(height) + ',U008 attr:BORDER_MODE:CONSTANT,0'),
    #('Remap_S16_nearest',                         'org.khronos.openvx.remap image:' + str(width) + ',' + str(height) + \
    # ',U008 remap:' + str(width) + ',' + str(height) + ',' + str(width) + ',' + str(height) + ' !NEAREST_NEIGHBOR image:' + str(width) + ',' + str(height) + ',U008'),
    #('Remap_S16_nearest_constant',                'org.khronos.openvx.remap image:' + str(width) + ',' + str(height) + \
    # ',U008 remap:' + str(width) + ',' + str(height) + ',' + str(width) + ',' + str(height) + ' !NEAREST_NEIGHBOR image:' + str(width) + ',' + str(height) + ',U008 attr:BORDER_MODE:CONSTANT,0'),
    #('Remap_S16_bilinear',                        'org.khronos.openvx.remap image:' + str(width) + ',' + str(height) + \
    # ',U008 remap:' + str(width) + ',' + str(height) + ',' + str(width) + ',' + str(height) + ' !BILINEAR image:' + str(width) + ',' + str(height) + ',U008'),
    #('Remap_S16_bilinear_constant',               'org.khronos.openvx.remap image:' + str(width) + ',' + str(height) + \
    # ',U008 remap:' + str(width) + ',' + str(height) + ',' + str(width) + ',' + str(height) + ' !BILINEAR image:' + str(width) + ',' + str(height) + ',U008 attr:BORDER_MODE:CONSTANT,0'),
    # vision kernels
    ('FastCorners_XY_U8_NoSupression',            'org.khronos.openvx.fast_corners uniform-image:' + str(width) + ',' + str(height) + \
    ',U008 scalar:FLOAT32,80.0 scalar:BOOL,1 array:KEYPOINT,1000 scalar:SIZE,0'),
    ('FastCorners_XY_U8_Supression',              'org.khronos.openvx.fast_corners uniform-image:' + str(width) + ',' + str(height) + \
    ',U008 scalar:FLOAT32,80.0 scalar:BOOL,0 array:KEYPOINT,1000 scalar:SIZE,0'),
    ('Canny_3x3_L1Norm',                          'org.khronos.openvx.canny_edge_detector uniform-image:' + str(width) + ',' + str(height) + \
    ',U008,0xab threshold:RANGE,U008,U008:INIT,80,100 scalar:INT32,3 !NORM_L1 image:' + str(width) + ',' + str(height) + ',U008'),
    ('Canny_3x3_L2Norm',                          'org.khronos.openvx.canny_edge_detector uniform-image:' + str(width) + ',' + str(height) + \
    ',U008,0xab threshold:RANGE,U008,U008:INIT,80,100 scalar:INT32,3 !NORM_L2 image:' + str(width) + ',' + str(height) + ',U008'),
]

#  Popular Video Sizes
# { 2160p: 3840x2160, 1440p: 2560x1440, 1080p: 1920x1080, 720p: 1280x720, 480p: 854x480, 360p: 640x360, 240p: 426x240 }
openvxNodeTestConfig = [
    # absdiff U8 - U8
    ('absdiff-240p-u8', 'org.khronos.openvx.absdiff image:426,240,U008 image:426,240,U008 image:426,240,U008'),
    ('absdiff-360p-u8', 'org.khronos.openvx.absdiff image:640,360,U008 image:640,360,U008 image:640,360,U008'),
    ('absdiff-480p-u8', 'org.khronos.openvx.absdiff image:854,480,U008 image:854,480,U008 image:854,480,U008'),
    ('absdiff-720p-u8', 'org.khronos.openvx.absdiff image:1280,720,U008 image:1280,720,U008 image:1280,720,U008'),
    ('absdiff-1080p-u8', 'org.khronos.openvx.absdiff image:1920,1080,U008 image:1920,1080,U008 image:1920,1080,U008'),
    ('absdiff-1440p-u8', 'org.khronos.openvx.absdiff image:2560,1440,U008 image:2560,1440,U008 image:2560,1440,U008'),
    ('absdiff-2160p-u8', 'org.khronos.openvx.absdiff image:3840,2160,U008 image:3840,2160,U008 image:3840,2160,U008'),
    # absdiff S16 - S16
    ('absdiff-240p-s16', 'org.khronos.openvx.absdiff image:426,240,S016 image:426,240,S016 image:426,240,S016'),
    ('absdiff-360p-s16', 'org.khronos.openvx.absdiff image:640,360,S016 image:640,360,S016 image:640,360,S016'),
    ('absdiff-480p-s16', 'org.khronos.openvx.absdiff image:854,480,S016 image:854,480,S016 image:854,480,S016'),
    ('absdiff-720p-s16', 'org.khronos.openvx.absdiff image:1280,720,S016 image:1280,720,S016 image:1280,720,S016'),
    ('absdiff-1080p-s16', 'org.khronos.openvx.absdiff image:1920,1080,S016 image:1920,1080,S016 image:1920,1080,S016'),
    ('absdiff-1440p-s16', 'org.khronos.openvx.absdiff image:2560,1440,S016 image:2560,1440,S016 image:2560,1440,S016'),
    ('absdiff-2160p-s16', 'org.khronos.openvx.absdiff image:3840,2160,S016 image:3840,2160,S016 image:3840,2160,S016')
]


# check arguments
if hardwareMode not in ('CPU', 'GPU'):
    print("ERROR: OpenVX Hardware supported - CPU or GPU]")
    exit(-1)
if listTest not in ('NO', 'YES'):
    print(
        "ERROR: List Vision Performance Tests options supported - [NO or YES]")
    exit(-1)
if backendType not in ('CPU', 'HIP', 'OCL'):
    print("ERROR: OpenVX Backends supported - CPU or HIP or OCL]")
    exit(-1)
if profilingOption not in ('NO', 'YES'):
    print("ERROR: Profiling options supported - [NO or YES]")
    exit(-1)
if perfCounters not in ('NO', 'YES'):
    print("ERROR: perf_counters options supported - [NO or YES]")
    exit(-1)
if profilingOption == "NO" and perfCounters == "YES":
    print("ERROR: To collect Performance counters both profiling and perfCounters must be YES")
    exit(-1)
if not 0 <= testFilter <= len(openvxNodes):
    print(
        "\nERROR: Vision Performance Filter not in range - [1 - %d]\n" % (len(openvxNodes)))
    exit(-1)
if not 1 <= numFrames <= 10000:
    print("\nERROR: Vision Test Number of Frames not in range - [1 - 10000]\n")
    exit(-1)
if not 1 <= width <= 7680:
    print("\nERROR: image width not in range - [1 - 7680]\n")
    exit(-1)
if not 1 <= height <= 7680:
    print("\nERROR: image width not in range - [1 - 7680]\n")
    exit(-1)

if hardwareMode == "CPU":
    if backendType != "CPU" or profilingOption != "NO":
        print("For hardware_mode=CPU, the backend_type must be 'CPU' and profiling must be 'NO'")
        exit(-1)
if hardwareMode == "GPU":
    if backendType == "CPU":
        print("For hardware_mode=GPU, the backend_type must be either 'HIP' or 'OCL'")
        exit(-1)

backendTypeValue = 0
if backendType == "OCL":
    backendTypeValue = 0
elif backendType == "HIP":
    backendTypeValue = 1

# List Vision Functionality tests
if listTest == 'YES':
    print(" %-5s - %-30s\n" % ('Test ID', 'Test Name'))
    for i in range(len(openvxNodes)):
        nodeName, nodeFormat = openvxNodes[i]
        print("   %-5d - %-30s\n" % ((i+1), nodeName))
    exit(0)

print("\nMIVisionX runVisionTests V-"+__version__+"\n")

# RunVX Application
runVX_exe = runvxDir+'/runvx'
RunVXapp = os.path.abspath(runVX_exe)
scriptPath = os.path.dirname(os.path.realpath(__file__))
if(os.path.isfile(RunVXapp)):
    print("STATUS: RunVX path - "+RunVXapp)
else:
    print("\nERROR: RunVX Executable Not Found\n")
    exit(-1)

# Get cwd
cwd = os.getcwd()

# OpenVX Performance Tests
print("\nrunVisionTests - OpenVX Node Performance\n")
outputDirectory = 'openvx_node_results'
if not os.path.exists(outputDirectory):
    os.makedirs(outputDirectory)
else:
    shutil.rmtree(outputDirectory)
    os.makedirs(outputDirectory)

# Option A - All cases / single case with GPU profiling

nodeList = []
case_num_list = []


def multiCaseProfilerOCL(nodeList, case_num_list):
    for i in nodeList:
        nodeName, nodeFormat = openvxNodes[i]
        echo1 = 'Running OpenVX Node - '+nodeName
        os.system('echo '+echo1 +
                ' | tee -a openvx_node_results/nodePerformanceOutput.log')
        os.system('mkdir '+cwd+'/rocprof_vision_tests_outputs/case_'+str(i+1))
        if perfCounters == "YES":
            print('rocprof -i rocprof_counters.txt -o "rocprof_vision_tests_outputs/case_'+str(i+1)+'/output_case_'+str(i+1)+'.csv" --basenames on --timestamp on --stats '+RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
                hardwareMode+' -dump-profile node '+nodeFormat)
            os.system('rocprof -i rocprof_counters.txt -o "rocprof_vision_tests_outputs/case_'+str(i+1)+'/output_case_'+str(i+1)+'.csv" --basenames on --timestamp on --stats '+RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
                hardwareMode+' -dump-profile node '+nodeFormat)
        else:
            print('rocprof -o "rocprof_vision_tests_outputs/case_'+str(i+1)+'/output_case_'+str(i+1)+'.csv" --basenames on --timestamp on --stats '+RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
                hardwareMode+' -dump-profile node '+nodeFormat)
            os.system('rocprof -o "rocprof_vision_tests_outputs/case_'+str(i+1)+'/output_case_'+str(i+1)+'.csv" --basenames on --timestamp on --stats '+RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
                hardwareMode+' -dump-profile node '+nodeFormat)

        print("\n")

    RESULTS_DIR = "rocprof_vision_tests_outputs"
    print("RESULTS_DIR = " + RESULTS_DIR)
    CONSOLIDATED_FILE = RESULTS_DIR + "/consolidated_results.stats.csv"
    new_file = open(CONSOLIDATED_FILE, 'w')
    new_file.write(
        '"OCL Kernel Name","Name","Calls","TotalDurationNs","AverageNs","Percentage"\n')

    for case_num in case_num_list:
        nodeName, nodeFormat = openvxNodes[case_num-1]
        CASE_RESULTS_DIR = RESULTS_DIR + "/case_" + str(case_num)
        print("CASE_RESULTS_DIR = " + CASE_RESULTS_DIR)
        CASE_FILE_PATH = CASE_RESULTS_DIR + \
            "/output_case_" + str(case_num) + ".stats.csv"
        print("CASE_FILE_PATH = " + CASE_FILE_PATH)
        try:
            case_file = open(CASE_FILE_PATH, 'r')
            for line in case_file:
                print(line)
                if line.startswith('"OpenVX_kernel'):
                    new_file.write('Ocl_'+nodeName+","+line)
            case_file.close()
        except IOError:
            print("Unable to open case results")
            continue

    new_file.close()
    os.system('chown $USER:$USER '+RESULTS_DIR +
            '/consolidated_results.stats.csv')

    try:
        import pandas as pd
        pd.options.display.max_rows = None
        df = pd.read_csv(CONSOLIDATED_FILE)
        df["AverageMs"] = df["AverageNs"] / 1000000
        dfPrint = df.drop(['Name', 'Percentage'], axis=1)
        dfPrint["OCL Kernel Name"] = dfPrint.iloc[:, 0].str.lstrip("Ocl_")
        print(dfPrint)
    except ImportError:
        print("\nPandas not available! Results of GPU profiling experiment are available in " + CONSOLIDATED_FILE)
    except IOError:
        print("Unable to open results in " + CONSOLIDATED_FILE)


def multiCaseProfilerHIP(nodeList, case_num_list):
    for i in nodeList:
        nodeName, nodeFormat = openvxNodes[i]
        echo1 = 'Running OpenVX Node - '+nodeName
        os.system('echo '+echo1 +
                ' | tee -a openvx_node_results/nodePerformanceOutput.log')
        os.system('mkdir '+cwd+'/rocprof_vision_tests_outputs/case_'+str(i+1))
        if perfCounters == "YES":
            print('rocprof -i rocprof_counters.txt -o "rocprof_vision_tests_outputs/case_'+str(i+1)+'/output_case_'+str(i+1)+'.csv" --basenames on --timestamp on --stats  '+RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
                hardwareMode+' -dump-profile node '+nodeFormat)
            os.system('rocprof -i rocprof_counters.txt -o "rocprof_vision_tests_outputs/case_'+str(i+1)+'/output_case_'+str(i+1)+'.csv" --basenames on --timestamp on --stats  '+RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
                hardwareMode+' -dump-profile node '+nodeFormat)
        else:
            print('rocprof -o "rocprof_vision_tests_outputs/case_'+str(i+1)+'/output_case_'+str(i+1)+'.csv" --basenames on --timestamp on --stats  '+RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
                hardwareMode+' -dump-profile node '+nodeFormat)
            os.system('rocprof -o "rocprof_vision_tests_outputs/case_'+str(i+1)+'/output_case_'+str(i+1)+'.csv" --basenames on --timestamp on --stats  '+RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
                hardwareMode+' -dump-profile node '+nodeFormat)
        print("\n")

    RESULTS_DIR = "rocprof_vision_tests_outputs"
    print("RESULTS_DIR = " + RESULTS_DIR)
    CONSOLIDATED_FILE = RESULTS_DIR + "/consolidated_results.stats.csv"
    new_file = open(CONSOLIDATED_FILE, 'w')
    new_file.write(
        '"HIP Kernel Name","Calls","TotalDurationNs","AverageNs","Percentage"\n')

    for case_num in case_num_list:
        nodeName, nodeFormat = openvxNodes[case_num-1]
        CASE_RESULTS_DIR = RESULTS_DIR + "/case_" + str(case_num)
        print("CASE_RESULTS_DIR = " + CASE_RESULTS_DIR)
        CASE_FILE_PATH = CASE_RESULTS_DIR + \
            "/output_case_" + str(case_num) + ".stats.csv"
        print("CASE_FILE_PATH = " + CASE_FILE_PATH)
        try:
            case_file = open(CASE_FILE_PATH, 'r')
            for line in case_file:
                print(line)
                if line.startswith('"Hip'):
                    new_file.write(line)
            case_file.close()
        except IOError:
            print("Unable to open case results")
            continue

    new_file.close()
    os.system('chown $USER:$USER '+RESULTS_DIR +
            '/consolidated_results.stats.csv')

    try:
        import pandas as pd
        pd.options.display.max_rows = None
        df = pd.read_csv(CONSOLIDATED_FILE)
        df["AverageMs"] = df["AverageNs"] / 1000000
        dfPrint = df.drop(['Percentage'], axis=1)
        dfPrint["HIP Kernel Name"] = dfPrint.iloc[:, 0].str.lstrip("Hip_")
        print(dfPrint)
    except ImportError:
        print("\nPandas not available! Results of GPU profiling experiment are available in " + CONSOLIDATED_FILE)
    except IOError:
        print("Unable to open results in " + CONSOLIDATED_FILE)


if testFilter == 0:
    totalCount = len(openvxNodes[:])
    nodeList = range(totalCount)
    case_num_list = range(1, totalCount + 1, 1)
else:
    totalCount = testFilter
    nodeList = range((totalCount - 1), totalCount, 1)
    case_num_list = range(totalCount, totalCount + 1, 1)

if profilingOption == "YES":

    os.system('rm -rvf '+cwd+'/rocprof_vision_tests_outputs')
    os.system('mkdir '+cwd+'/rocprof_vision_tests_outputs')

    if backendType == "OCL":
        multiCaseProfilerOCL(nodeList=nodeList, case_num_list=case_num_list)
    else:
        multiCaseProfilerHIP(nodeList=nodeList, case_num_list=case_num_list)

# Option B - All cases / single case without GPU profiling
else:
    for i in nodeList:
        nodeName, nodeFormat = openvxNodes[i]
        echo1 = 'Running OpenVX Node - '+str(i+1)+':'+nodeName
        os.system('echo '+echo1 +
                ' | tee -a openvx_node_results/nodePerformanceOutput.log')
        print(RunVXapp+' -frames:'+str(numFrames)+' -affinity:' +
            hardwareMode+' -dump-gdf -dump-profile node '+nodeFormat)
        os.system(RunVXapp+' -frames:'+str(numFrames)+' -affinity:'+hardwareMode +
                ' -dump-gdf -dump-profile node '+nodeFormat+' | tee -a openvx_node_results/nodePerformanceOutput.log')
        print("\n")
    orig_stdout = sys.stdout
    sys.stdout = open('openvx_node_results/nodePerformance.md', 'a')
    echo_1 = '| OpenVX Node                             |Frames Count| tmp (ms) | avg (ms) | min (ms) | max (ms) |'
    print(echo_1)
    echo_2 = '|-----------------------------------------|------------|----------|----------|----------|----------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    if hardwareMode == 'CPU':
        runAwk_csv = r'''awk 'BEGIN { node = "xxx"; } /Running OpenVX Node - / { node = $5; } /CPU,GRAPH/ { printf("| %-39s | %10d | %8.3f | %8.3f | %8.3f | %8.3f |\n", node, $1, $2, $3, $4, $5) }' openvx_node_results/nodePerformanceOutput.log | tee -a openvx_node_results/nodePerformance.md'''
    elif hardwareMode == 'GPU':
        runAwk_csv = r'''awk 'BEGIN { node = "xxx"; } /Running OpenVX Node - / { node = $5; } /GPU,GRAPH/ { printf("| %-39s | %10d | %8.3f | %8.3f | %8.3f | %8.3f |\n", node, $1, $2, $3, $4, $5) }' openvx_node_results/nodePerformanceOutput.log | tee -a openvx_node_results/nodePerformance.md'''
    os.system(runAwk_csv)

    # get data
    platform_name = platform.platform()
    if os.path.exists('/usr/bin/yum'):
        if not "centos" in platform_name or not "redhat" in platform_name:
            platform_name = platform_name+'-CentOS-RedHat'
    elif os.path.exists('/usr/bin/apt-get'):
        if not "Ubuntu" in platform_name:
            platform_name = platform_name+'-Ubuntu'
    elif os.path.exists('/usr/bin/zypper'):
        if not "SLES" in platform_name:
            platform_name = platform_name+'-SLES'
    else:
        print("\nMIVisionX Vision Test on "+platform_name+" is unsupported")
        print("MIVisionX Vision Test Supported on: Ubuntu 22/24; CentOS 7/8; RedHat 8/9; & SLES 15 SP3")
        print("\nMIVisionX Vision Test on "+platform_name+" is unreliable")

    platform_name_fq = shell('hostname --all-fqdns')
    platform_ip = shell('hostname -I')[0:-1]  # extra trailing space

    file_dtstr = datetime.now().strftime("%Y%m%d")
    reportFilename = 'vision_report_%s_%s_%s.md' % (
        platform_name, file_dtstr, backendType)
    report_dtstr = datetime.now().strftime("%Y-%m-%d %H:%M:%S %Z")
    sys_info = shell('inxi -c0 -S')

    cpu_info = shell('inxi -c0 -C')
    # cpu_info = cpu_info.split('\n')[0]  # strip out clock speeds

    gpu_info = shell('inxi -c0 -G')
    # gpu_info = gpu_info.split('\n')[0]  # strip out X info

    memory_info = shell('inxi -c 0 -m')
    board_info = shell('inxi -c0 -M')

    lib_tree = shell('ldd '+RunVXapp)
    lib_tree = strip_libtree_addresses(lib_tree)

    # Write Report
    with open(reportFilename, 'w') as f:
        f.write("MIVisionX - OpenVX Function Report\n")
        f.write("================================\n")
        f.write("\n")

        f.write("Generated: %s\n" % report_dtstr)
        f.write("\n")

        f.write("Platform: %s (%s)\n" % (platform_name_fq, platform_ip))
        f.write("--------\n")
        f.write("\n")

        write_formatted(sys_info, f)
        write_formatted(cpu_info, f)
        write_formatted(gpu_info, f)
        write_formatted(board_info, f)
        write_formatted(memory_info, f)

        f.write("\n\nBenchmark Report\n")
        f.write("--------\n")
        f.write("\n")
        f.write("### Hardware: %s\n" % hardwareMode)
        f.write("\n")
        with open('openvx_node_results/nodePerformance.md') as benchmarkFile:
            for line in benchmarkFile:
                f.write("%s" % line)
        f.write("\n")
        f.write("\n")
        f.write("Dynamic Libraries Report\n")
        f.write("-----------------\n")
        f.write("\n")
        write_formatted(lib_tree, f)
        f.write("\n")

        f.write(
            "\n\n---\n**Copyright AMD ROCm MIVisionX 2018 - 2024 -- runVisionTests.py V-"+__version__+"**\n")
        f.write("\n")

        # report file
        reportFileDir = os.path.abspath(reportFilename)
        print("\nSTATUS: Output Report File - "+reportFileDir)

print("\nrunVisionTests.py completed - V:"+__version__+"\n")



# Copyright (c) 2015 - 2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

import argparse
import os
import shutil
import sys
import traceback
import platform
if sys.version_info[0] < 3:
    import commands
else:
    import subprocess
from datetime import datetime
from subprocess import Popen, PIPE

__copyright__ = "Copyright 2018 - 2024, AMD MIVisionX - Neural Net Test Full Report"
__license__ = "MIT"
__version__ = "2.1.0"
__email__ = "mivisionx.support@amd.com"
__status__ = "Shipping"
    
# error check calls
def ERROR_CHECK(call):
    status = call
    if(status != 0):
        print('ERROR_CHECK failed with status:'+str(status))
        traceback.print_stack()
        exit(status)


def shell(cmd):
    p = Popen(cmd, shell=True, stdout=PIPE, stderr=PIPE)
    output = p.communicate()[0][0:-1]
    return output


def write_formatted(output, f):
    f.write("````\n")
    f.write("%s\n\n" % output)
    f.write("````\n")


def write_lines_as_table(header, lines, f):
    for h in header:
        f.write("|%s" % h)
    f.write("|\n")

    for h in header:
        f.write("|:---")
    f.write("|\n")

    for l in lines:
        fields = l.split()
        for field in fields:
            f.write("|%s" % field)
        f.write("|\n")


def strip_libtree_addresses(lib_tree):
    return lib_tree


def script_info():
    print("\nMIVisionX runNeuralNetworkTests V-"+__version__+"\n")
    print(
        "--profiler_mode       - NN Profile Mode: optional (default:0 [range:0 - 9])")
    print("    --profiler_mode 0 -- Run All Tests")
    print("    --profiler_mode 1 -- Run caffe2nnir2openvx No Fuse flow")
    print("    --profiler_mode 2 -- Run caffe2nnir2openvx Fuse flow")
    print("    --profiler_mode 3 -- Run caffe2nnir2openvx FP16 flow")
    print("    --profiler_mode 4 -- Run onnx2nnir2openvx No Fuse flow")
    print("    --profiler_mode 5 -- Run onnx2nnir2openvx Fuse flow")
    print("    --profiler_mode 6 -- Run onnx2nnir2openvx FP16 flow")
    print("    --profiler_mode 7 -- Run nnef2nnir2openvx No Fuse flow")
    print("    --profiler_mode 8 -- Run nnef2nnir2openvx Fuse flow")
    print("    --profiler_mode 9 -- Run nnef2nnir2openvx FP16 flow")
    print(
        "--profiler_level      - NN Profile Batch Size in powers of 2: optional (default:7 [range:1 - N])")
    print(
        "--miopen_find         - MIOPEN_FIND_ENFORCE mode: optional (default:1 [range:1 - 5])")


# models to run - add new models `modelname` , c, h, w
caffeModelConfig = [
    ('caffe-mnist', 1, 28, 28)
]

onnxModelConfig = [
    ('onnx-squeezenet', 3, 224, 224)
]

nnefModelConfig = [
    ('nnef-mnist', 1, 28, 28)
]

# REPORT
reportConfig = [
    ('CAFFE no fused OPs', 'caffe2nnir2openvx_noFuse_profile.md'),
    ('CAFFE fused OPs', 'caffe2nnir2openvx_Fuse_profile.md'),
    ('CAFFE fp16', 'caffe2nnir2openvx_FP16_profile.md'),
    ('ONNX no fused OPs', 'onnx2nnir2openvx_noFuse_profile.md'),
    ('ONNX fused Ops', 'onnx2nnir2openvx_Fuse_profile.md'),
    ('ONNX fp16', 'onnx2nnir2openvx_FP16_profile.md'),
    ('NNEF no fused OPs', 'nnef2nnir2openvx_noFuse_profile.md'),
    ('NNEF fused OPs', 'nnef2nnir2openvx_Fuse_profile.md'),
    ('NNEF fp16', 'nnef2nnir2openvx_FP16_profile.md')
]

# Import arguments
parser = argparse.ArgumentParser()
parser.add_argument('--profiler_mode',      type=int, default=0,
                    help='NN Profile Mode - optional (default:0 [range:0 - 9])')
parser.add_argument('--profiler_level',     type=int, default=7,
                    help='NN Profile Batch Size in powers of 2 - optional (default:7 [range:1 - N])')
parser.add_argument('--miopen_find',        type=int, default=1,
                    help='MIOPEN_FIND_ENFORCE mode - optional (default:1 [range:1 - 5])')
parser.add_argument('--test_info',          type=str, default='no',
                    help='Show test info - optional (default:no [options:no/yes])')
parser.add_argument('--backend_type',       type=str, default='HIP',
                    help='Backend type - optional (default:HIP [options:HOST/HIP/OCL])')
parser.add_argument('--install_directory',    type=str, default='/opt/rocm',
                    help='MIVisionX Install Directory - optional')
parser.add_argument('--reinstall', 	type=str, default='ON',
                    help='Remove previous setup and reinstall - optional (default:OFF) [options:ON/OFF]')
args = parser.parse_args()

profileMode = args.profiler_mode
profileLevel = args.profiler_level
miopenFind = args.miopen_find
testInfo = args.test_info
backendType = args.backend_type
ROCM_PATH = args.install_directory
reinstall = args.reinstall.upper()

platformInfo = platform.platform()

returnStatus = 0

# check arguments
if not 0 <= profileMode <= 9:
    print(
        "\nERROR: NN Profile Mode not in range - [0 - 9]\n")
    exit()
if not 1 <= profileLevel <= 10:
    print(
        "\nERROR: NN Profile Batch Size in powers of 2 not in range - [1 - 10]\n")
    exit()
if not 1 <= miopenFind <= 5:
    print(
        "\nERROR: MIOPEN_FIND_ENFORCE not in range - [1 - 5]\n")
    exit()
if testInfo not in ('no', 'yes'):
    print("ERROR: Show test info options supported - [no or yes]")
    script_info()
    exit()

if testInfo == 'yes':
    script_info()
    exit()

# check backend
if backendType not in ('HOST', 'HIP', 'OCL'):
    print("ERROR: Backends supported - HOST or HIP or OCL]")
    exit()

if backendType == 'HOST':
    print("ERROR: HOST Backend currently NOT Supported [Supported: OCL/HIP]")
    exit()

if reinstall not in ('OFF', 'ON'):
    print(
        "ERROR: Re-Install Option Not Supported - [Supported Options: OFF or ON]\n")
    parser.print_help()
    exit()

# override default path if env path set 
if "ROCM_PATH" in os.environ:
    ROCM_PATH = os.environ.get('ROCM_PATH')
print("ROCm PATH set to -- "+ROCM_PATH+"\n")

# check install
runVX_exe = ROCM_PATH+'/bin/runvx'
if (os.path.isfile(runVX_exe)):
    print("STATUS: MIVisionX Install Path Found - "+ROCM_PATH)
else:
    print("\nERROR: MIVisionX Install Path Not Found\n")
    exit()

print("\nMIVisionX runNeuralNetworkTests V-"+__version__+"\n")

# check for Scripts
scriptPath = os.path.dirname(os.path.realpath(__file__))
modelCompilerDir = os.path.expanduser(
    ROCM_PATH+'/libexec/mivisionx/model_compiler/python')
pythonScript = modelCompilerDir+'/caffe_to_nnir.py'
modelCompilerScript = os.path.abspath(pythonScript)
if (os.path.isfile(modelCompilerScript)):
    print("\nMIVisionX Neural Net Tests on "+platformInfo+"\n")
    print("STATUS: Model Compiler Scripts Used from - "+modelCompilerDir+"\n")
else:
    print("ERROR: Model Compiler Scripts Not Found at - "+modelCompilerDir)
    print("ERROR: MIVisionX Not Installed, install MIVisionX and rerun")
    exit()

# Install Model Compiler Deps
modelCompilerDeps = os.path.expanduser('~/.mivisionx-model-compiler-deps')
linuxCMake = 'cmake'

# check os version
os_info_data = 'NOT Supported'
if os.path.exists('/etc/os-release'):
    with open('/etc/os-release', 'r') as os_file:
        os_info_data = os_file.read().replace('\n', ' ')
        os_info_data = os_info_data.replace('"', '')
        
# Linux Packages
inferenceDebianPackages = [
    'inxi',
    'python3-dev',
    'python3-pip'
]

inferenceRPMPackages = [
    'inxi',
    'python3-devel',
    'python3-pip'
]

# pip3 versions
pipNumpyVersion = "numpy~=1.23.0"
pipProtoVersion= "protobuf~=3.12.4"
pipONNXVersion = "onnx~=1.12.0"
pipFutureVersion = "future~=1.0.0"
pipPytzVersion = "pytz~=2022.1"
pipGoogleVersion = "google~=3.0.0"
pipNNEFVersion = "nnef~=1.0.7"

# Debian pip3 packages
if "VERSION_ID=24" in os_info_data:
    pipNumpyVersion = "numpy~=2.0.0"
    pipONNXVersion = "onnx~=1.16.0"
    pipProtoVersion= "protobuf~=3.20.2"

pip3InferencePackagesDebian = [
    str(pipFutureVersion),
    str(pipPytzVersion),
    str(pipGoogleVersion),
    str(pipNumpyVersion),
    str(pipProtoVersion),
    str(pipONNXVersion),
    str(pipNNEFVersion)
]

# RPM pip3 packages
pipONNXversion = "onnx~=1.11.0"
if "VERSION_ID=8" in os_info_data:
    pipNumpyVersion = "numpy==1.19.5"
    pipNNEFversion = "protobuf==3.12.4" # TBD: NO NNEF Package for RHEL 8
if "NAME=SLES" in os_info_data:
    pipNumpyVersion = "numpy==1.19.5"
    pipProtoVersion= "protobuf==3.19.5"
    pipNNEFversion = "protobuf==3.19.5" # TBD: NO NNEF Package for SLES

pip3InferencePackagesRPM = [
    str(pipFutureVersion),
    str(pipPytzVersion),
    str(pipGoogleVersion),
    str(pipNumpyVersion),
    str(pipProtoVersion),
    str(pipONNXVersion),
    str(pipNNEFVersion)
]

# Delete previous install
if os.path.exists(modelCompilerDeps) and reinstall == 'ON':
    os.system('sudo -v')
    os.system('sudo rm -rf '+modelCompilerDeps)
    print("\nMIVisionX runNeuralNetworkTests: Removing Previous Install -- " +modelCompilerDeps+"\n")

if not os.path.exists(modelCompilerDeps):
    print("STATUS: Model Compiler Deps Install - "+modelCompilerDeps+"\n")
    # sudo requirement check
    sudoLocation = ''
    userName = ''
    if sys.version_info[0] < 3:
        status, sudoLocation = commands.getstatusoutput("which sudo")
        if sudoLocation != '/usr/bin/sudo':
            status, userName = commands.getstatusoutput("whoami")
    else:
        status, sudoLocation = subprocess.getstatusoutput("which sudo")
        if sudoLocation != '/usr/bin/sudo':
            status, userName = subprocess.getstatusoutput("whoami")
    
    # setup for Linux
    linuxSystemInstall = ''
    linuxCMake = 'cmake'
    linuxSystemInstall_check = ''
    linuxFlag = ''
    sudoValidate = 'sudo -v'
    if "centos" in os_info_data or "redhat" in os_info_data:
        linuxSystemInstall = 'yum -y'
        linuxSystemInstall_check = '--nogpgcheck'
        if "VERSION_ID=7" in os_info_data:
            linuxCMake = 'cmake3'
            sudoValidate = 'sudo -k'
            platformInfo = platformInfo+'-redhat-7'
        elif "VERSION_ID=8" in os_info_data:
            platformInfo = platformInfo+'-redhat-8'
        elif "VERSION_ID=9" in os_info_data:
            platformInfo = platformInfo+'-redhat-9'
        else:
            platformInfo = platformInfo+'-redhat-centos-undefined-version'
    elif "Ubuntu" in os_info_data:
        linuxSystemInstall = 'apt-get -y'
        linuxSystemInstall_check = '--allow-unauthenticated'
        linuxFlag = '-S'
        if "VERSION_ID=20" in os_info_data:
            platformInfo = platformInfo+'-Ubuntu-20'
        elif "VERSION_ID=22" in os_info_data:
            platformInfo = platformInfo+'-Ubuntu-22'
        elif "VERSION_ID=24" in os_info_data:
            platformInfo = platformInfo+'-Ubuntu-24'
        else:
            platformInfo = platformInfo+'-Ubuntu-undefined-version'
    elif "SLES" in os_info_data:
        linuxSystemInstall = 'zypper -n'
        linuxSystemInstall_check = '--no-gpg-checks'
        platformInfo = platformInfo+'-SLES'
    elif "Mariner" in os_info_data:
        linuxSystemInstall = 'tdnf -y'
        linuxSystemInstall_check = '--nogpgcheck'
        platformInfo = platformInfo+'-Mariner'
    else:
        print("\nMIVisionX runNeuralNetworkTests.py on "+platformInfo+" is unsupported\n")
        print("\nMIVisionX Setup Supported on: Ubuntu 22/24; CentOS 7/8; RedHat 8/9; & SLES 15 SP5\n")
        exit()

    if userName == 'root':
        ERROR_CHECK(os.system(linuxSystemInstall+' update'))
        ERROR_CHECK(os.system(linuxSystemInstall+' install sudo'))

    os.makedirs(modelCompilerDeps)
    os.system('sudo -v')
    if "Ubuntu" in platformInfo:
        for i in range(len(inferenceDebianPackages)):
            ERROR_CHECK(os.system('sudo '+linuxFlag+' '+linuxSystemInstall +
                        ' '+linuxSystemInstall_check+' install -y '+ inferenceDebianPackages[i]))
        for i in range(len(pip3InferencePackagesDebian)):
                            ERROR_CHECK(os.system('pip3 install '+ pip3InferencePackagesDebian[i]))
    else:
        for i in range(len(inferenceRPMPackages)):
            ERROR_CHECK(os.system('sudo '+linuxFlag+' '+linuxSystemInstall +
                        ' '+linuxSystemInstall_check+' install -y '+ inferenceRPMPackages[i]))
        for i in range(len(pip3InferencePackagesRPM)):
                            ERROR_CHECK(os.system('pip3 install '+ pip3InferencePackagesRPM[i]))
        if "SLES" in platformInfo or "Mariner" in platformInfo or "redhat-8" in platformInfo:
            ERROR_CHECK(os.system('mkdir -p '+modelCompilerDeps+'/nnef-deps'))
            ERROR_CHECK(os.system(
                '(cd '+modelCompilerDeps+'/nnef-deps; git clone -b nnef-v1.0.0 https://github.com/KhronosGroup/NNEF-Tools.git)'))
            ERROR_CHECK(os.system(
                '(cd '+modelCompilerDeps+'/nnef-deps/NNEF-Tools/parser/cpp; mkdir -p build && cd build; '+linuxCMake+' ..; make -j$(nproc); sudo make install)'))
            ERROR_CHECK(os.system(
                '(cd '+modelCompilerDeps+'/nnef-deps/NNEF-Tools/parser/python; sudo python3 setup.py install)'))
else:
    print("STATUS: Model Compiler Deps Pre-Installed - "+modelCompilerDeps+"\n")
    if "centos-7" in platformInfo:
        linuxCMake = 'cmake3'

currentWorkingDirectory = os.getcwd()

# Create working directory
outputDirectory = currentWorkingDirectory+'/vx_nn_test'
if not os.path.exists(outputDirectory):
    os.makedirs(outputDirectory)
else:
    shutil.rmtree(outputDirectory)
    os.makedirs(outputDirectory)

# run caffe2nnir2openvx no fuse flow
if profileMode == 0 or profileMode == 1:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/caffeNoFuse'
    os.makedirs(outputDirectory)
    for i in range(len(caffeModelConfig)):
        modelName, channel, height, width = caffeModelConfig[i]
        print("\n caffe2nnir2openvx with NO FUSED Operations -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/caffe_to_nnir.py '+scriptPath+'/models/' +
                    modelName+'/model.caffemodel . --input-dims '+x+','+str(channel)+','+str(height)+','+str(width)+')')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --fuse-ops 0 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/caffe_no_fuse_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/caffe_no_fuse_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_no_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_noFuse_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_no_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_noFuse_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/caffe2nnir2openvx_noFuse_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_no_fuse_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_noFuse_profile.md'''
    os.system(runAwk_md)

# run caffe2nnir2openvx with fuse flow
if profileMode == 0 or profileMode == 2:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/caffeFuse'
    os.makedirs(outputDirectory)
    for i in range(len(caffeModelConfig)):
        modelName, channel, height, width = caffeModelConfig[i]
        print("\n caffe2nnir2openvx with FUSED Operations -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/caffe_to_nnir.py '+scriptPath+'/models/' +
                    modelName+'/model.caffemodel . --input-dims '+x+','+str(channel)+','+str(height)+','+str(width)+')')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --fuse-ops 1 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/caffe_fuse_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/caffe_fuse_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_Fuse_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_Fuse_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/caffe2nnir2openvx_Fuse_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_fuse_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_Fuse_profile.md'''
    os.system(runAwk_md)

# run caffe2nnir2openvx with fp16 flow
if profileMode == 0 or profileMode == 3:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/caffeFP16'
    os.makedirs(outputDirectory)
    for i in range(len(caffeModelConfig)):
        modelName, channel, height, width = caffeModelConfig[i]
        print("\n caffe2nnir2openvx FP16 -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/caffe_to_nnir.py '+scriptPath+'/models/' +
                    modelName+'/model.caffemodel . --input-dims '+x+','+str(channel)+','+str(height)+','+str(width)+')')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --convert-fp16 1 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/caffe_fp16_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/caffe_fp16_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_fp16_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_FP16_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_fp16_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_FP16_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/caffe2nnir2openvx_FP16_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe_fp16_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/caffe2nnir2openvx_FP16_profile.md'''
    os.system(runAwk_md)

# run onnx2nnir2openvx no fuse flow
if profileMode == 0 or profileMode == 4:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/onnxNoFuse'
    os.makedirs(outputDirectory)
    for i in range(len(onnxModelConfig)):
        modelName, channel, height, width = onnxModelConfig[i]
        print("\n onnx2nnir2openvx with NO FUSED Operations -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/onnx_to_nnir.py '+scriptPath+'/models/' +
                    modelName+'/model.onnx . --input-dims '+x+','+str(channel)+','+str(height)+','+str(width)+')')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --fuse-ops 0 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/onnx_no_fuse_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/onnx_no_fuse_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_no_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_noFuse_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_no_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_noFuse_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/onnx2nnir2openvx_noFuse_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_no_fuse_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_noFuse_profile.md'''
    os.system(runAwk_md)

# run onnx2nnir2openvx with fuse flow
if profileMode == 0 or profileMode == 5:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/onnxFuse'
    os.makedirs(outputDirectory)
    for i in range(len(onnxModelConfig)):
        modelName, channel, height, width = onnxModelConfig[i]
        print("\n onnx2nnir2openvx with FUSED Operations -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/onnx_to_nnir.py '+scriptPath+'/models/' +
                    modelName+'/model.onnx . --input-dims '+x+','+str(channel)+','+str(height)+','+str(width)+')')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --fuse-ops 1 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/onnx_fuse_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/onnx_fuse_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_Fuse_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_Fuse_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/onnx2nnir2openvx_Fuse_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_fuse_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_Fuse_profile.md'''
    os.system(runAwk_md)

# run onnx2nnir2openvx with fp16 flow
if profileMode == 0 or profileMode == 6:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/onnxFP16'
    os.makedirs(outputDirectory)
    for i in range(len(onnxModelConfig)):
        modelName, channel, height, width = onnxModelConfig[i]
        print("\n onnx2nnir2openvx FP16 -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/onnx_to_nnir.py '+scriptPath+'/models/' +
                    modelName+'/model.onnx . --input-dims '+x+','+str(channel)+','+str(height)+','+str(width)+')')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --convert-fp16 1 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/onnx_fp16_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/onnx_fp16_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_fp16_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_FP16_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_fp16_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_FP16_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/onnx2nnir2openvx_FP16_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx_fp16_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/onnx2nnir2openvx_FP16_profile.md'''
    os.system(runAwk_md)

# run nnef2nnir2openvx no fuse flow
if profileMode == 0 or profileMode == 7:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/nnefNoFuse'
    os.makedirs(outputDirectory)
    for i in range(len(nnefModelConfig)):
        modelName, channel, height, width = nnefModelConfig[i]
        print("\n nnef2nnir2openvx with No FUSED Operations -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/nnef_to_nnir.py '+scriptPath+'/models/' +
                    modelName+' . )')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --fuse-ops 0 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/nnef_no_fuse_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/nnef_no_fuse_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_no_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_noFuse_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_no_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_noFuse_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/nnef2nnir2openvx_noFuse_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_no_fuse_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_noFuse_profile.md'''
    os.system(runAwk_md)

# run nnef2nnir2openvx fuse flow
if profileMode == 0 or profileMode == 8:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/nnefFuse'
    os.makedirs(outputDirectory)
    for i in range(len(nnefModelConfig)):
        modelName, channel, height, width = nnefModelConfig[i]
        print("\n nnef2nnir2openvx with FUSED Operations -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/nnef_to_nnir.py '+scriptPath+'/models/' +
                    modelName+' . )')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --fuse-ops 1 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/nnef_fuse_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/nnef_fuse_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_Fuse_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_fuse_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_Fuse_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/nnef2nnir2openvx_Fuse_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_fuse_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_Fuse_profile.md'''
    os.system(runAwk_md)

# run nnef2nnir2openvx FP16 flow
if profileMode == 0 or profileMode == 9:
    outputDirectory = currentWorkingDirectory+'/vx_nn_test/nnefFP16'
    os.makedirs(outputDirectory)
    for i in range(len(nnefModelConfig)):
        modelName, channel, height, width = nnefModelConfig[i]
        print("\n nnef2nnir2openvx with FP16 Operations -- "+modelName+"\n")
        modelBuildDir = outputDirectory+'/nnir_build_'
        for x in range(profileLevel):
            x = 2**x
            x = str(x)
            print("\n"+modelName+" - Batch size "+x)
            os.system('(cd '+outputDirectory +
                    '; mkdir -p nnir_build_'+x+')')
            os.system('(cd '+modelBuildDir+x+'; python3 '+modelCompilerDir+'/nnef_to_nnir.py '+scriptPath+'/models/' +
                    modelName+' . )')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_update.py --convert-fp16 1 . .)')
            os.system('(cd '+modelBuildDir+x+'; python3 ' +
                    modelCompilerDir+'/nnir_to_openvx.py . .)')
            os.system('(cd '+modelBuildDir+x+'; '+linuxCMake+' .; make)')
            os.system('echo '+modelName+' - Batch size '+x+'  | tee -a ' +
                    currentWorkingDirectory+'/vx_nn_test/nnef_fp16_output.log')
            os.system('(cd '+modelBuildDir+x+'; MIOPEN_FIND_ENFORCE='+str(miopenFind) +
                    ' ./anntest weights.bin | tee -a '+currentWorkingDirectory+'/vx_nn_test/nnef_fp16_output.log)')
            annTestResults = shell(
                '(cd '+modelBuildDir+x+'; ./anntest weights.bin)')
            annTestResults = annTestResults.decode()
            if annTestResults.find("successful") == -1:
                returnStatus = -1

    runAwk_csv = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s,%3d,%8.3f ms,%8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_fp16_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_FP16_profile.csv'''
    os.system(runAwk_csv)
    runAwk_txt = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("%-16s %3d %8.3f ms %8.3f ms\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_fp16_output.log > ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_FP16_profile.txt'''
    os.system(runAwk_txt)

    orig_stdout = sys.stdout
    sys.stdout = open(
        currentWorkingDirectory+'/vx_nn_test/nnef2nnir2openvx_FP16_profile.md', 'a')
    echo_1 = '|      Model Name      | Batch Size | Time/Batch (ms) | Time/Image (ms) |'
    print(echo_1)
    echo_2 = '|----------------------|------------|-----------------|-----------------|'
    print(echo_2)
    sys.stdout = orig_stdout
    print(echo_1)
    print(echo_2)
    runAwk_md = r'''awk 'BEGIN { net = "xxx"; bsize = 1; } / - Batch size/ { net = $1; bsize = $5; } /average over 100 iterations/ { printf("|%-22s|%-12d|%-17.3f|%-17.3f|\n", net, bsize, $4, $4/bsize) }' ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef_fp16_output.log | tee -a ''' + \
        currentWorkingDirectory+'''/vx_nn_test/nnef2nnir2openvx_FP16_profile.md'''
    os.system(runAwk_md)

# get system data
platform_name = platform.platform()
if os.path.exists('/usr/bin/yum'):
    if "centos" not in platform_name or "redhat" not in platform_name:
        platformInfo = platform_name+'-CentOS-RedHat'
elif os.path.exists('/usr/bin/apt-get'):
    if "Ubuntu" not in platform_name:
        platform_name = platform_name+'-Ubuntu'
elif os.path.exists('/usr/bin/zypper'):
    if "SLES" not in platform_name:
        platform_name = platform_name+'-SLES'
else:
    print("\nMIVisionX Neural Network Test on "+platform_name+" is unsupported")
    print("MIVisionX Neural Network Test Supported on: Ubuntu 22/24; CentOS 7/8; RedHat 8/9; & SLES 15 SP3")
    print("\nMIVisionX Neural Network Test on "+platform_name+" is unreliable")

platform_name_fq = shell('hostname --all-fqdns')
platform_ip = shell('hostname -I')[0:-1]  # extra trailing space

file_dtstr = datetime.now().strftime("%Y%m%d")
reportFilename = 'inference_report_%s_%s_%s.md' % (
    backendType, platform_name, file_dtstr)
report_dtstr = datetime.now().strftime("%Y-%m-%d %H:%M:%S %Z")
sys_info = shell('inxi -c0 -S')

cpu_info = shell('inxi -c0 -C')
cpu_info = cpu_info.rstrip()  # strip out clock speeds

gpu_info = shell('inxi -c0 -G')
gpu_info = gpu_info.rstrip()  # strip out X info

memory_info = shell('inxi -c 0 -m')
board_info = shell('inxi -c0 -M')

lib_tree = shell('ldd -v '+ROCM_PATH+'/lib/libvx_nn.so')
lib_tree = strip_libtree_addresses(lib_tree)

vbios = shell('(cd /opt/rocm/bin/; ./rocm-smi -v)')

rocmInfo = shell('(cd /opt/rocm/bin/; ./rocm-smi -a)')

rocm_packages = shell('dpkg-query -W | grep rocm')
rocm_packages = rocm_packages.splitlines()

# Write Report
with open(reportFilename, 'w') as f:
    f.write("MIVisionX - ML Inference Report\n")
    f.write("================================\n")
    f.write("\n")

    f.write("Generated: %s\n" % report_dtstr)
    f.write("\n")

    f.write("Platform: %s (%s)\n" % (platform_name_fq, platform_ip))
    f.write("--------\n")
    f.write("\n")

    write_formatted(sys_info, f)
    write_formatted(cpu_info, f)
    write_formatted(gpu_info, f)
    write_formatted(board_info, f)
    write_formatted(memory_info, f)

    f.write("\n\nBenchmark Report\n")
    f.write("--------\n")
    f.write("\n")

    if profileMode == 0:
        for i in range(len(reportConfig)):
            modelType, reportFile = reportConfig[i]
            f.write("\n### MODEL FORMAT: %s\n" % modelType)
            with open(currentWorkingDirectory+'/vx_nn_test/'+reportFile) as benchmarkFile:
                for line in benchmarkFile:
                    f.write("%s" % line)
    else:
        modelType, reportFile = reportConfig[profileMode - 1]
        f.write("\n### MODEL FORMAT: %s\n" % modelType)
        with open(currentWorkingDirectory+'/vx_nn_test/'+reportFile) as benchmarkFile:
            for line in benchmarkFile:
                f.write("%s" % line)

    f.write("\n")

    f.write("ROCm Package and Version Report\n")
    f.write("-------------\n")
    f.write("\n")
    write_lines_as_table(['Package', 'Version'], rocm_packages, f)
    f.write("\n\n\n")

    f.write("Vbios Report\n")
    f.write("-------------\n")
    f.write("\n")
    write_formatted(vbios, f)
    f.write("\n")
    f.write("ROCm Device Info Report\n")
    f.write("-------------\n")
    f.write("\n")
    write_formatted(rocmInfo, f)
    f.write("\n")

    f.write("Dynamic Libraries Report\n")
    f.write("-----------------\n")
    f.write("\n")
    write_formatted(lib_tree, f)
    f.write("\n")

    f.write("\n\n---\n**Copyright AMD ROCm MIVisionX 2018 - 2024 -- runNeuralNetworkTests.py V-"+__version__+"**\n")

# report file
reportFileDir = os.path.abspath(reportFilename)
print("\nSTATUS: Output Report File - "+reportFileDir)

print("\nrunNeuralNetworkTests.py completed - V:"+__version__+"\n")

exit(returnStatus)



# Copyright (c) 2018 - 2024 Advanced Micro Devices, Inc. All rights reserved.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE 
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

from __future__ import print_function
from __future__ import division
from __future__ import absolute_import
from __future__ import unicode_literals
from future import standard_library
standard_library.install_aliases()
from builtins import *
from builtins import range
from past.utils import old_div
import os, sys
import onnx
from onnx import onnx_pb
from onnx import numpy_helper
import nnir as ir

onnx2ir_attr = {
    'axis' : 'axis',
    'axes'  : 'axes',
    'perm' : 'axes',
    'broadcast' : 'broadcast',
    'keepdims' : 'keepdims',
    'kernel_shape' : 'kernel_shape',
    'pads' : 'pads',
    'strides' : 'strides',
    'dilations' : 'dilations',
    'group' : 'group',
    'epsilon' : 'epsilon',
    'alpha' : 'alpha',
    'beta' : 'beta',
    'transA' : 'transA',
    'transB' : 'transB',
    'bias' : 'bias',
    'size' : 'size',
    'split' : 'split',
    'shape' : 'shape',
    'min' : 'min',
    'max' : 'max',
    'to' : 'to', 
    'center_point_box' : 'center_point_box',
    'value' : 'value',
    'largest' : 'largest',
    'sorted' : 'sorted',
}

onnx2ir_op_type = { 
    'Conv'               : 'conv',
    'ConvTranspose'      : 'conv_transpose',
    'BatchNormalization' : 'batch_norm',
    'AveragePool'        : 'avg_pool',
    'MaxPool'            : 'max_pool',
    'Relu'               : 'relu',
    'Sum'                : 'sum',
    'Add'                : 'add',
    'Sub'                : 'sub',
    'Mul'                : 'mul',
    'MatMul'             : 'matmul',
    'Gemm'               : 'gemm',
    'LRN'                : 'lrn',
    'Concat'             : 'concat',
    'LeakyRelu'          : 'leaky_relu',
    'Sigmoid'            : 'sigmoid',
    'GlobalAveragePool'  : 'global_avg_pool',
    'Softmax'            : 'softmax',
    'Reshape'            : 'reshape',
    'Squeeze'            : 'squeeze',
    'Unsqueeze'          : 'unsqueeze',
    'Transpose'          : 'transpose',
    'Flatten'            : 'flatten',
    'Identity'           : 'copy',
    'Min'                : 'min',
    'Max'                : 'max',
    'Div'                : 'div',
    'Exp'                : 'exp',
    'Log'                : 'log',
    'ReduceMean'         : 'global_avg_pool',
    'Clip'               : 'clamp',
    'Cast'               : 'cast',
    'Shape'              : 'shape',  
    'ArgMax'             : 'argmax',
    'NonMaxSuppression'  : 'nms',
    'Constant'           : 'constant',
    'Gather'             : 'gather',
    'TopK'               : 'topk',
    'ReduceMin'          : 'reduce_min',
    'Tile'               : 'tile',
    'Less'               : 'less',
    'Greater'            : 'greater',
    'LessOrEqual'        : 'less_equal',
    'GreaterOrEqual'     : 'greater_equal',
    'Equal'              : 'equal'
}

onnx2ir_data_type = [
    "UND_", "F032", "U008", "I008", "U016", "I016", "I032", "I064",
    "STR_", "BOOL", "F016", "F064", "U032", "U064", "C064", "C128"
]

def onnx_name_to_ir_name(name):
    return '_'.join('_'.join(('_'.join(('_'.join(name.split('/')).split('-')))).split(':')).split('.'))

def onnx_node_to_ir_attr(node):
    global onnx2ir_attr
    attr = ir.IrAttr()
    for item in node.attribute:
        if item.name in onnx2ir_attr:
            name = onnx2ir_attr[item.name]
            if item.HasField('f'):
                attr.set(name,float(item.f))
            elif item.HasField('i'):
                attr.set(name,int(item.i))
            elif item.HasField('s'):
                attr.set(name,item.s)
            elif item.HasField('t'):
                attr.set(name,numpy_helper.to_array(item.t))
            elif len(item.floats):
                attr.set(name,list(item.floats))
            elif len(item.ints):
                attr.set(name,[int(v) for v in list(item.ints)])
            elif len(item.strings):
                attr.set(name,list(item.strings))
            else:
                raise ValueError("Unsupported ONNX attribute: {}".format(item))
    if attr.is_set('output_padding'):
        output_padding = attr.get('output_padding')
        kernel_shape = attr.get('kernel_shape')
        if (kernel_shape[0] <= 1) or (kernel_shape[1] <= 1) or \
           ((output_padding[0] % (kernel_shape[0] - 1)) != 0) or \
           ((output_padding[1] % (kernel_shape[1] - 1)) != 0):
            raise ValueError("Unsupported ONNX value for output_padding attribute")
        dilations = [old_div(output_padding[0], (kernel_shape[0] - 1)) + 1, old_div(output_padding[1], (kernel_shape[1] - 1)) + 1]
        attr.set('dilations', dilations)       
    if node.op_type == 'MatMul':
        attr.set('beta', 0.0)
    return attr

def onnx_node_to_ir_node(onnx_node):
    global onnx2ir_op_type
    node = ir.IrNode()
    if onnx_node.op_type in onnx2ir_op_type:
        type = onnx2ir_op_type[onnx_node.op_type]
    else:
        print('ERROR: ONNX operation "%s" not supported yet' % (onnx_node.op_type))
        sys.exit(1)
    node.set(type, [onnx_name_to_ir_name(name) for name in onnx_node.input], \
                   [onnx_name_to_ir_name(name) for name in onnx_node.output], \
                   onnx_node_to_ir_attr(onnx_node))
    return node

def onnx_tensor_info_to_data(info, dims):
    tensor = ir.IrTensor()
    tensor.setName(onnx_name_to_ir_name(info.name))
    tensor.setInfo(onnx2ir_data_type[info.data_type], [int(x) for x in dims])
    return tensor

def onnx_value_info_to_data(info, dims):
    tensor = ir.IrTensor()
    tensor.setName(onnx_name_to_ir_name(info.name))
    tensor.setInfo(onnx2ir_data_type[info.type.tensor_type.elem_type], [int(x) for x in dims])
    return tensor

def onnx_graph_to_ir_graph(onnx_graph):
    graph = ir.IrGraph(False)
    initializerList = []
    shapeList = []
    inputUser = False
    tensorAliasList = {}

    for onnx_node in onnx_graph.node:
        if onnx_node.op_type == 'Reshape' and len(onnx_node.input) == 2:
            for tensor in onnx_graph.initializer:
                tensorName = onnx_name_to_ir_name(tensor.name)
                if tensor.name == onnx_node.input[1] and tensorName not in shapeList:
                    shapeList.append(tensorName)
                    graph.addVariable(onnx_tensor_info_to_data(tensor,numpy_helper.to_array(tensor)))
                    raw_data = numpy_helper.to_array(tensor)
                    graph.addBinary(tensorName, raw_data)
        if onnx_node.op_type == 'Dropout':
            tensorAliasList[onnx_node.output[0]] = onnx_node.input[0]
        else:
            for i in range(len(onnx_node.input)):
                if onnx_node.input[i] in tensorAliasList:
                    onnx_node.input[i] = tensorAliasList[onnx_node.input[i]]
            node = onnx_node_to_ir_node(onnx_node)
            graph.addNode(node)
    for tensor in onnx_graph.initializer:
        tensorName = onnx_name_to_ir_name(tensor.name)
        if tensorName not in shapeList:
            initializerList.append(tensorName)
            graph.addVariable(onnx_tensor_info_to_data(tensor, tensor.dims))
            raw_data = numpy_helper.to_array(tensor)
            graph.addBinary(tensorName, raw_data)
    for tensor in onnx_graph.input:
        if onnx_name_to_ir_name(tensor.name) not in initializerList and onnx_name_to_ir_name(tensor.name) not in shapeList:
            input_dims = [int(x.dim_value) for x in tensor.type.tensor_type.shape.dim]
            if (len(sys.argv) > 3) and (sys.argv[3] == "--input_dims"):
                if (x == 0 or x is None or x == '?' for x in input_dims):
                    input_dims = sys.argv[4].split(',')
                    inputUser = True
            graph.addInput(onnx_value_info_to_data(tensor, input_dims))
    for tensor in onnx_graph.output:
        output_dims = [int(x.dim_value) for x in tensor.type.tensor_type.shape.dim]
        if (x == 0 or x is None or x == '?' for x in output_dims):
            if inputUser is True:
                output_dims[0] = input_dims[0]
        while len(output_dims) != 4:
            output_dims.append(1)
        graph.addOutput(onnx_value_info_to_data(tensor, output_dims))
    graph.updateLocals()
    return graph

def onnx2ir(model, output_folder, node_type_append):
    # get graph from ONNX model
    if isinstance(model, str):
        onnx_model = onnx.load(model)
    elif isinstance(model, onnx.ModelProto):
        onnx_model = model
    else:
        raise TypeError("Model must be file path to .onnx file or onnx loaded model")
    graph = onnx_graph_to_ir_graph(onnx_model.graph)
    graph.toFile(output_folder, node_type_append)

def main():
    if len(sys.argv) < 3:
        print('Usage: python onnx_to_nnir.py <onnxModel> <nnirOutputFolder> [--input_dims n,c,h,w (optional)] [--node_type_append 0/1 (optional: appends node type to output tensor name)]')
        sys.exit(1)
    onnxFileName = sys.argv[1]
    outputFolder = sys.argv[2]
    #appends node type to output tensor name. 
    node_type_append = 0
    pos = 4
    while pos < len(sys.argv)  and len(sys.argv) > 3 and sys.argv[pos][:2] == '--':
        if sys.argv[pos] == '--node_type_append':
            node_type_append = int(sys.argv[pos+1])
            pos = pos + 2
        elif sys.argv[pos] == '--input_dims':
            #input_dims = sys.argv[pos+1]
            pos = pos + 2
    print('loading ONNX model from %s ...' % (onnxFileName))
    onnx_model_proto = onnx_pb.ModelProto()
    if not os.path.isfile(onnxFileName):
        print('ERROR: unable to open: ' + onnxFileName)
        sys.exit(1)
    onnx_model_proto.ParseFromString(open(onnxFileName, 'rb').read())
    print('converting to IR model in %s ...' % (outputFolder))
    onnx2ir(onnx_model_proto, outputFolder, node_type_append)

if __name__ == '__main__':
    main()